#!/usr/bin/env python
# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
from __future__ import (unicode_literals, division, absolute_import, print_function)
from calibre_plugins.ki_notes.localize import (_, gettext)

__license__   = 'GPL v3'
__copyright__ = '2013, Axel Walthelm'

import os, re

from PyQt4.Qt import QWidget, QIcon, QHBoxLayout, QLabel, QLineEdit
from PyQt4.QtGui import QRegExpValidator
from PyQt4.QtCore import QRegExp

from calibre.utils.config import JSONConfig
from calibre_plugins.ki_notes.lib.CssStyles import (KINOTE_DEMO_TEXT, KINOTE_STYLES)

# This is where all preferences for this plugin will be stored
# Remember that this name (i.e. plugins/ki_notes) is also
# in a global namespace, so make it as unique as possible.
# You should always prefix your config file name with plugins/,
# so as to ensure you dont accidentally clobber a calibre config file
prefs = JSONConfig('plugins/ki_notes')

# Set defaults
prefs.defaults['_is_installed'] = False
prefs.defaults['style'] = 'PROOFREADER'
prefs.defaults['style_tabstops'] = True
prefs.defaults['style_highlighting'] = True
prefs.defaults['style_anchors'] = True
prefs.defaults['style_id_timestamp'] = False
prefs.defaults['save_postfix'] = ' + KiNotes'
prefs.defaults['save_backups'] = 9

class ConfigWidget(QWidget):

    def __init__(self):
        QWidget.__init__(self)
#        self.l = QHBoxLayout()
#        self.setLayout(self.l)
#        
#        self.label = QLabel('Hello world &message:')
#        self.l.addWidget(self.label)
#
#        self.msg = QLineEdit(self)
#        self.msg.setText(prefs['hello_world_msg'])
#        self.l.addWidget(self.msg)
#        self.label.setBuddy(self.msg)
        self._top_window_icon = None
        self.iconArrowUp = I('arrow-up.png')
        self.iconArrowDown = I('arrow-down.png')
        if (self.iconArrowUp and os.path.isfile(self.iconArrowUp) and
            self.iconArrowDown and os.path.isfile(self.iconArrowDown)):
            self.iconArrowUp = QIcon(self.iconArrowUp)
            self.iconArrowDown = QIcon(self.iconArrowDown)
        else:
            self.iconArrowUp = self.iconArrowDown = None
        import calibre_plugins.ki_notes.config_ui as config_ui
        config_ui._ = _
        self.ui = config_ui.Ui_Form()
        self.ui.setupUi(self)
        
        self.ui.pushButton_StyleAdvanced.toggled.connect(self.advancedOptionsShown)
        showAdvanced = not prefs['style_highlighting'] or not prefs['style_anchors']
        self.ui.pushButton_StyleAdvanced.setChecked(not showAdvanced)
        self.ui.pushButton_StyleAdvanced.setChecked(showAdvanced)
        
        for id_, name, css in KINOTE_STYLES:
            self.ui.comboBox_Style.addItem(name, id_)
        styleIds = [i for i,n,c in KINOTE_STYLES]
        if prefs['style'] in styleIds:
            self.ui.comboBox_Style.setCurrentIndex( styleIds.index(prefs['style']) )
        self.ui.checkBox_StyleTabStops.setChecked( prefs['style_tabstops'] )
        self.ui.checkBox_StyleHighlighting.setChecked( prefs['style_highlighting'] )
        self.ui.checkBox_StyleStartEndAnchors.setChecked( not prefs['style_anchors'] )
        self.ui.checkBox_StyleStartEndAnchors.setChecked( prefs['style_anchors'] )
        self.ui.checkBox_StyleIdTimestamp.setChecked( prefs['style_id_timestamp'] )
        self.ui.lineEdit_SavePostfix.setText( prefs['save_postfix'] )
        self.ui.spinBox_SaveBackups.setValue( prefs['save_backups'] )
        
        self.ui.comboBox_Style.currentIndexChanged.connect(self.updateStylePreview)
        self.ui.checkBox_StyleHighlighting.toggled.connect(self.updateStylePreview)
        self.ui.checkBox_StyleStartEndAnchors.toggled.connect(self.updateStylePreview)
        self.ui.checkBox_StyleTabStops.toggled.connect(self.updateStylePreview)
        self.updateStylePreview()
        
        self.ui.lineEdit_SavePostfix.textChanged.connect(self.updateSavePreview)
        self.ui.spinBox_SaveBackups.valueChanged.connect(self.updateSavePreview)
        self.updateSavePreview()
        
        self.ui.lineEdit_SavePostfix.setValidator(
            QRegExpValidator(QRegExp(r'[^\000-\037<>:"/\\|?*]*')))

    def showEvent(self, event):
        QWidget.showEvent(self, event)
        from calibre_plugins.ki_notes.ui import pluginIcon
        if pluginIcon:
            try:
                window = self
                while not window.isWindow() and window.parentWidget():
                    window = window.parentWidget()
                if window.isWindow():
                    self._top_window_icon = (window, window.windowIcon())
                    window.setWindowIcon(pluginIcon)
            except:
                import traceback
                traceback.print_exc()
    
    def hideEvent(self, event):
        QWidget.hideEvent(self, event)
        try:
            if self._top_window_icon:
                self._top_window_icon[0].setWindowIcon(self._top_window_icon[1])
        except:
            import traceback
            traceback.print_exc()
            
    def advancedOptionsShown(self, isShow):
        print("KiNotes: advancedOptionsShown %r" % isShow)
        if self.iconArrowUp and self.iconArrowDown:
            self.ui.pushButton_StyleAdvanced.setIcon(self.iconArrowDown if isShow else self.iconArrowUp)
            
    def updateStylePreview(self):
        print("KiNotes: updateStylePreview")
        styleId = str(self.ui.comboBox_Style.itemData(self.ui.comboBox_Style.currentIndex()).toString())
        print("KiNotes: styleId = %r" % styleId)
        css = [c for i,n,c in KINOTE_STYLES if i==styleId][0]
        print("KiNotes: css %r %r" % (len(css), css[:20]))
        body = KINOTE_DEMO_TEXT
        if not self.ui.checkBox_StyleHighlighting.isChecked():
            body = re.sub(r'\sclass=[\'"]KiNoteMark[^\'"]*"', ' ', body)
        if not self.ui.checkBox_StyleStartEndAnchors.isChecked():
            body = re.sub(r'<span\s[^>]*class=[\'"]KiNote[^M].*?</span>', '', body)
        if not self.ui.checkBox_StyleTabStops.isChecked():
            body = re.sub(r'\stabindex="[0-9]+"', ' ', body)
        elif not self.ui.checkBox_StyleStartEndAnchors.isChecked():
            body = re.sub(r'(<span class="KiNoteMark[^"]*")>', r'\1 tabindex="1">', body)
        html = '<html><head><style>%s</style></head><body style="font-family:arial;">%s</body></html>' % (css, body)
        self.ui.webView_StylePreview.setHtml(html)
        
    def updateSavePreview(self):
        postfix = unicode(self.ui.lineEdit_SavePostfix.text())
        backups = self.ui.spinBox_SaveBackups.value()
        def nbsp(text):
            return text.replace(u' ', u'&nbsp;')
        def info(text, color = '#666666'):
            return nbsp(u' ')+u'<font color="%s"><em>%s</em></font>' % (color, nbsp(text))
        def warn(text):
            return info(text, '#990000')
        epub = _("Alice in Wonderland - Lewis Carroll.epub")
        c_old = _("original file")
        c_new = _("new file")
        c_rep = _("new file replaces original file")
        c_bck = _("backup file")
        c_bck_new = _("newest backup file")
        c_bck_old = _("oldest backup file")
        lines = [nbsp(epub) + (info(c_old) if postfix else (info(c_new) if backups else warn(c_rep)))]
        if postfix:
            epub = re.sub(ur"\.epub$", postfix + u".epub", epub)
            lines.append(nbsp(epub)+info(c_new))
        idx = 0
        while idx < backups:
            idx += 1
            if backups >= 16 and idx == 13:
                idx = backups-2
                lines.append(info(_("[...]")))
            lines.append( nbsp(re.sub(ur"\.epub$", u" (%d).epub" % idx, epub)) )
            if idx == 1:
                lines[-1] += info(c_bck_new if backups > 1 else c_bck)
            elif idx == backups:
                lines[-1] += info(c_bck_old)
        self.ui.textEdit_SavePreview.setHtml('<br>'.join(lines))

    def save_settings(self):
        prefs['style'] = str(self.ui.comboBox_Style.itemData(self.ui.comboBox_Style.currentIndex()).toString())
        prefs['style_tabstops'] = self.ui.checkBox_StyleTabStops.isChecked()
        prefs['style_highlighting'] = self.ui.checkBox_StyleHighlighting.isChecked()
        prefs['style_anchors'] = self.ui.checkBox_StyleStartEndAnchors.isChecked()
        prefs['style_id_timestamp'] = self.ui.checkBox_StyleIdTimestamp.isChecked()
        prefs['save_postfix'] = unicode(self.ui.lineEdit_SavePostfix.text())
        prefs['save_backups'] = self.ui.spinBox_SaveBackups.value()
