# -*- coding: utf-8 -*-
assert '…' == '\xe2\x80\xa6', "file encoding error"

# Copyright 2013 Axel Walthelm
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

from localize import (_, gettext)
from collections import namedtuple

KINOTE_CLASSES = frozenset((
    'KiNoteStart',
    'KiNoteStarts',
    'KiNoteEnd',
    'KiNoteEnds',
    'KiNoteFuzzyStart',
    'KiNoteFuzzyEnd',
    'KiNoteBookmark',
    'KiNoteMark0',
    'KiNoteMark',
    'KiNoteMarkX'))
    
KINOTE_DEMO_TEXT = _(
"""
<p>This example text contains a <%s>highlight<%s>, a <%s>note<%s>, and a bookmark.<%s></p>
<p><%s>Highlights and <%s>notes<%s> may overlap.<%s></p>
<p>Sometimes an annotation <%s>can not <%s>be placed<%s> perfectly,<%s> e.g. because the EPUB text was edited. Then an uncertainty region is marked and the original text is given.</p>
""") % (
    '''span class="KiNoteStart" title="'1"><b>[</b></span><span class="KiNoteMark"''',
    '''/span><span class="KiNoteEnd" tabindex="1" title="'1"><b>]</b></span''',
    '''span class="KiNoteStarts" title="1"><b>[</b></span><span class="KiNoteMark"''',
    '''/span><span class="KiNoteEnds" tabindex="2" title="1"><b>]</b><i>%s</i></span''' % _("this is the note"),
    '''span class="KiNoteBookmark" tabindex="3" title="#1"><b>[]</b></span''',
    '''span class="KiNoteStart" title="'2"><b>[</b></span><span class="KiNoteMark"''',
    '''span class="KiNoteStarts" title="2"><b>[</b></span></span><span class="KiNoteMarkX"''',
    '''/span><span class="KiNoteMark"><span class="KiNoteEnd" tabindex="4" title="'2"><b>]</b></span''',
    '''/span><span class="KiNoteEnds" tabindex="5" title="2"><b>]</b><i>%s</i></span''' % _('is "notes" darker?'),
    '''span class="KiNoteFuzzyStart" title="'3"><b>{</b></span><span class="KiNoteMark0"''',
    '''span class="KiNoteStart" title="'3"><b>[</b></span></span><span class="KiNoteMark"''',
    '''/span><span class="KiNoteMark0"><span class="KiNoteEnd" tabindex="6" title="'3"><b>]</b></span''',
    '''/span><span class="KiNoteFuzzyEnd" tabindex="7" title="'3"><b>}</b><i>%s</i></span''' % _("ca<u>n't</u> <s>always</s> be placed <u>corr</u>ectly,"),
    )

Style = namedtuple('Stype', 'id name css')

KINOTE_STYLES = (

Style('PROOFREADER', _('Proof Reader'),
"""
.KiNoteStart,
.KiNoteStarts,
.KiNoteFuzzyStart,
.KiNoteEnd,
.KiNoteEnds,
.KiNoteFuzzyEnd,
.KiNoteBookmark {
    text-decoration: none;
    font-size: .83em;
    line-height: 0.5em;
    vertical-align: baseline;
    position: relative;
    top: -0.35em;
    border: 1px solid #888888;
    background-color: rgb(255,249,189);
}
.KiNoteStart,
.KiNoteEnd {
    background-color: rgb(255,222,189);
}
.KiNoteBookmark {
    background-color: rgb(235,255,189);
}
.KiNoteFuzzyStart,
.KiNoteFuzzyEnd {
    background-color: rgb(189,202,255);
}
.KiNoteStart,
.KiNoteStarts,
.KiNoteFuzzyStart {
    border-radius: 8px 0px 0px 8px;
    border-width: 1px 0px 1px 1px;
    padding-left: 1.5ex;
    padding-right: 3px;
    margin-right: 1px;
}
.KiNoteEnd,
.KiNoteEnds,
.KiNoteFuzzyEnd {
    border-radius: 0px 8px 8px 0px;
    border-width: 1px 1px 1px 0px;
    padding-right: 1.5ex;
    padding-left: 3px;
    margin-left: 1px;
}
.KiNoteBookmark {
    border-radius: 4px;
    padding-left: 2px;
    padding-right: 2px;
    margin-left: 1px;
    margin-right: 1px;
}
.KiNoteEnd:after,
.KiNoteEnds:after,
.KiNoteFuzzyEnd:after {
    content: ""; /* work around strange clipping at end of line */
}
.KiNoteStart[title]:focus:after,
.KiNoteStart[title]:active:after,
.KiNoteStarts[title]:focus:after,
.KiNoteStarts[title]:active:after,
.KiNoteFuzzyStart[title]:focus:after,
.KiNoteFuzzyStart[title]:active:after,
.KiNoteEnd[title]:focus:before,
.KiNoteEnd[title]:active:before {
    content: attr(title);
}
.KiNoteEnds[title]:focus:before,
.KiNoteEnds[title]:active:before,
.KiNoteFuzzyEnd[title]:focus:before,
.KiNoteFuzzyEnd[title]:active:before {
    content: attr(title) ": ";
}
.KiNoteBookmark:before{
    content: "#?";
}
.KiNoteBookmark[title]:before{
    content: attr(title);
}
.KiNoteStart>b,
.KiNoteEnd>b,
.KiNoteStarts>b,
.KiNoteEnds>b,
.KiNoteBookmark>b,
.KiNoteFuzzyStart>b,
.KiNoteFuzzyEnd>b {
    display: none;    
}
.KiNoteEnds>i,
.KiNoteFuzzyEnd>i {
    font-style: normal;
}
.KiNoteMark0 {
    background-color: #EEE;
}
.KiNoteMark {
    background-color: #CCC;
}
.KiNoteMarkX {
    background-color: #999;
}
"""),

Style('KINDLE', _('Kindle'),
"""
.KiNoteStart:focus:after,
.KiNoteStart:hover:after,
.KiNoteStarts:focus:after,
.KiNoteStarts:hover:after,
.KiNoteEnd:focus:after,
.KiNoteEnd:hover:after,
.KiNoteEnds,
.KiNoteFuzzyEnd>i,
.KiNoteFuzzyEnd:hover:before,
.KiNoteFuzzyEnd:focus:before,
.KiNoteBookmark {
    font-size: .83em;
	line-height: 0.5em;
    vertical-align: baseline;
	position: relative;
	top: -0.35em;
	border: 1px solid #888888;
    background-color: rgb(247,244,193);
	border-radius: 4px;
	padding-left: 2px;
	padding-right: 2px;
}
.KiNoteStart[title]:focus:after,
.KiNoteStart[title]:hover:after,
.KiNoteStarts[title]:focus:after,
.KiNoteStarts[title]:hover:after {
	content: attr(title);
	border-radius: 4px 0px 0px 4px;
	border-width: 1px 0px 1px 1px;
}
.KiNoteEnd[title]:focus:after,
.KiNoteEnd[title]:hover:after {
	content: attr(title);
	border-radius: 0px 4px 4px 0px;
	border-width: 1px 1px 1px 0px;
}
.KiNoteEnds>i {
	display: none;
}
.KiNoteEnds:before {
	content: attr(title);
}
.KiNoteEnds[title]:focus:before,
.KiNoteEnds[title]:hover:before {
	content: attr(title) ": ";
}
.KiNoteEnds[title]:focus>i,
.KiNoteEnds[title]:hover>i {
	display: inherit;
}
.KiNoteBookmark[title]:focus:after,
.KiNoteBookmark[title]:hover:after {
	content: " " attr(title);
}
.KiNoteStart:before,
.KiNoteStarts:before,
.KiNoteEnd:before {
    content: "\A0";
	font-size: 0;
}
.KiNoteStart,
.KiNoteStarts,
.KiNoteEnd {
    background-color: #CCC;
	padding-left: 2px;
}
.KiNoteFuzzyStart,
.KiNoteFuzzyEnd {
    background-color: #EEE;
	padding-left: 2px;
	padding-right: 2px;
}
/* CSS 2.1 will ignore the following CSS 3 rule */
.KiNoteFuzzyEnd:not(:empty):hover:before,
.KiNoteFuzzyEnd:not(:empty):focus:before {
	content: attr(title) ": ";
	border-radius: 4px 0px 0px 4px;
	border-width: 1px 0px 1px 1px;
	padding-left: 0px;
	padding-right: 0px;
}
.KiNoteFuzzyEnd:hover>i,
.KiNoteFuzzyEnd:focus>i {
	border-radius: 0px 4px 4px 0px;
	border-width: 1px 1px 1px 0px;
}
.KiNoteBookmark {
	margin-left: 1px;
	margin-right: 1px;
}
.KiNoteBookmark:before {
	content: "\A0\A0\A0";
	background: url(data:image/gif;base64,R0lGODlhGQAgAJEAAAAAAMDAwMzMzP///yH5BAkAAAMALAAAAAAZACAAAAioAAcIHEiwoMGDCBMeBMCwocOGAhQOeEgRokKGATJqzPgwIkKMCwEIsBgyQEgBIxl6LAjSoEqUJAm2ZCkSZUqRNE26rGkz5kQAOmna7MlQJtCQFYsOnGk0KQCjQSUOCKBUIFOJVJ8uPSpVYFaoXb1W/RkV69irCr9uLZv2LNeuaq2+lRqXbNipbtkmrIt2b967fOea1Sp3o+HDHAn/dMq4KeOkdyNLJhgQAA==) no-repeat center;
	background-origin: content-box;
    background-size: contain;
}
.KiNoteStart>b,
.KiNoteEnd>b,
.KiNoteStarts>b,
.KiNoteEnds>b,
.KiNoteBookmark>b,
.KiNoteFuzzyStart>b,
.KiNoteFuzzyEnd>b {
	display: none;
}
.KiNoteEnds>i,
.KiNoteFuzzyEnd>i {
	font-style: normal;
}
.KiNoteMark0 {
    background-color: #EEE;
}
.KiNoteMark {
    background-color: #CCC;
}
.KiNoteMarkX {
    background-color: #999;
}
"""),

Style('PLAIN', _('Plain Bold'),
"""
.KiNoteStart,
.KiNoteStarts,
.KiNoteEnd,
.KiNoteEnds,
.KiNoteFuzzyStart,
.KiNoteFuzzyEnd,
.KiNoteBookmark,
.KiNoteEnds>i,
.KiNoteFuzzyEnd>i {
    font-weight: bold;
    font-style: normal;
    color: black;
}
.KiNoteMark0 {
    background-color: #EEE;
}
.KiNoteMark {
    background-color: #CCC;
}
.KiNoteMarkX {
    background-color: #999;
}
"""),

Style('HIDDEN', _('Invisible KiNotes'),
"""
.KiNoteStart,
.KiNoteStarts,
.KiNoteEnd,
.KiNoteEnds,
.KiNoteFuzzyStart,
.KiNoteFuzzyEnd,
.KiNoteBookmark {
	display: none;
}
.KiNoteMark0,
.KiNoteMark,
.KiNoteMarkX {
}
"""),
)
