# -*- coding: utf-8 -*-
assert '…' == '\xe2\x80\xa6', "file encoding error"

# Copyright 2013 Axel Walthelm
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import re, platform, os, urllib

global _isWindows
_isWindows = platform.system() == 'Windows'

# url is ASCII (but may be converted to unicode)
# returned path is unicode
def localFilePathOfUri(url):
    if not url:
        return u''
    # on Windows: if url looks like a native file path, use it as is
    if _isWindows and re.match(r"[a-zA-Z]:[\\/]|[\\/]{2}", url):
        return unicode(url)
    if isinstance(url, unicode):
        url = url.encode('utf-8')
    else:
        url.decode('ascii') # check URI is ASCII; if not, convert it to unicode before calling!
    assert isinstance(url, bytes)
    url = url.strip()
    if url.startswith('"') and url.endswith('"') or url.startswith("'") and url.endswith("'"):
        url = url[1:-1]
    url = re.sub(r"file://localhost/", r"file:///", url, 1, re.IGNORECASE)
    if _isWindows:
        # if path starts with a drive designation, tolerate any number of / before it
        match = re.match(r"^file:/*([a-z])[:|]([\\/].*|)$", url, re.IGNORECASE)
        if match:
            url = r"file:///%s:%s" % (match.group(1).upper(), match.group(2) if match.group(2) else '/')
        else:
            # if host name is specified, we use network share syntax
            # if url has 4 leading /, consider it a network share too
            match = re.match(r"^file://(?://)?([^/]+)/(.*)$", url, re.IGNORECASE)
            if match:
                url = r"file://///%s/%s" % match.groups() # Firefox syntax
        # Windows on Mac with shared folders (is this a real case?)
        url = re.sub(r"^file:///Shared Folders/", r"file:///Z:/", url, 1, re.IGNORECASE)
        if not re.match(r"file:///", url, re.IGNORECASE):
            return u'' # not a valid file url
        return urllib.unquote(url[8:]).decode('utf-8').replace(u'/', u'\\')
    else:
        # no support for remote host file access;
        # if host is specified, we assume it is synonymous to localhost
        url = re.sub(ur"^file://[^/]+/", r"file:///", url, 1, re.IGNORECASE)
        if not re.match(r"file:///", url, re.IGNORECASE):
            return u'' # not a valid file url
        return urllib.unquote(url[7:]).decode('utf-8')

# url is ASCII (but may be converted to unicode)
# returned path is unicode if it exists, else it is None
def existingLocalFilePathOfUri(url):
    path = localFilePathOfUri(url)
    return path if os.path.isfile(path) else None

try:
    from os.path import samefile
except:
    # soon Python will support this in os.path for Windows and Unix;
    # for now we use:
    def samefile(f1, f2):
        return os.path.normcase(os.path.abspath(f1)) == os.path.normcase(os.path.abspath(f2))

if __debug__ and __name__ == '__main__':
    def _runTests():
        print "Test localFilePathOfUri"
        global _isWindows
        def test_localFilePathOfUri(url, expectedFilePath):
            path = localFilePathOfUri(url)
            if path != expectedFilePath:
                print "localFilePathOfUri(%r)" % url
                print "expected: %r" % expectedFilePath
                print "  but is: %r" % path
            assert isinstance(expectedFilePath, unicode)
            assert path == expectedFilePath
            assert isinstance(path, unicode)
            
        try:
            for isWin in (False, True):
                _isWindows = isWin
                test_localFilePathOfUri(None, u'')
                test_localFilePathOfUri('', u'')
                test_localFilePathOfUri(u'', u'')
                test_localFilePathOfUri('file', u'')
                test_localFilePathOfUri('file:', u'')
                test_localFilePathOfUri('file:/', u'')
                test_localFilePathOfUri('file://', u'')
                test_localFilePathOfUri(r'http://foo.com/file.txt', u'')
            
            _isWindows = False
            test_localFilePathOfUri('file:///', ur'/')
            test_localFilePathOfUri(r'file:///folder/file.txt', ur'/folder/file.txt')            
            test_localFilePathOfUri(r'"file:///folder/file.txt"', ur'/folder/file.txt')            
            test_localFilePathOfUri(r"'file:///folder/file.txt'", ur'/folder/file.txt')            
            test_localFilePathOfUri(r'file://localhost/folder/file.txt', ur'/folder/file.txt')
            test_localFilePathOfUri(r'file:///a/.', ur'/a/.')
            test_localFilePathOfUri(r'file:///a/b.', ur'/a/b.')
            test_localFilePathOfUri(r'file:///a/b.c', ur'/a/b.c')
            test_localFilePathOfUri(r'file:///a/.c', ur'/a/.c')
            test_localFilePathOfUri(r'file:///etc/hosts', ur'/etc/hosts')
            test_localFilePathOfUri(r'file:///%', u'/%')
            test_localFilePathOfUri(r'file:///%%', u'/%%')
            test_localFilePathOfUri(r'file:///%25', u'/%')
            
            # invalid on Mac, but translated systematically
            test_localFilePathOfUri(r'file:///:', u'/:')
            
            _isWindows = True
            test_localFilePathOfUri('file:///', u'')
            test_localFilePathOfUri(r'file:///d:', u'D:\\')
            test_localFilePathOfUri(r'file:///d|', u'D:\\')
            test_localFilePathOfUri(r'file:///d:/', u'D:\\')
            test_localFilePathOfUri(r'file:///d|/', u'D:\\')
            test_localFilePathOfUri(r'file:///a:/.', ur'A:\.')
            test_localFilePathOfUri(r'file:///a:/b.', ur'A:\b.')
            test_localFilePathOfUri(r'file:///a:/b.c', ur'A:\b.c')
            test_localFilePathOfUri(r'file:///a:/.c', ur'A:\.c')
            test_localFilePathOfUri(r'file:///C:/Autoexec.bat', ur'C:\Autoexec.bat')
            test_localFilePathOfUri(r'file:///C:/%', u'C:\%')
            test_localFilePathOfUri(r'file:///C:/%%', u'C:\%%')
            test_localFilePathOfUri(r'file:///C:/%25', u'C:\%')

            test_localFilePathOfUri(r'file:///d:/folder/file.txt', ur'D:\folder\file.txt')
            test_localFilePathOfUri(r'"file:///d:/folder/file.txt"', ur'D:\folder\file.txt')
            test_localFilePathOfUri(r"'file:///d:\folder\file.txt'", ur'D:\folder\file.txt')
            test_localFilePathOfUri(r'file://localhost/d:/folder/file.txt', ur'D:\folder\file.txt')
            test_localFilePathOfUri(r'file://d:/folder/file.txt', ur'D:\folder\file.txt')
            test_localFilePathOfUri(r'file:///d|/f1/f2/f3/file.txt', ur'D:\f1\f2\f3\file.txt')
            test_localFilePathOfUri(r'file://d|/f1/f2/f3/file.txt', ur'D:\f1\f2\f3\file.txt')
            test_localFilePathOfUri(r'file://laptop/My%20Documents/FileSchemeURIs.doc', ur'\\laptop\My Documents\FileSchemeURIs.doc')
            test_localFilePathOfUri(r'file:///C:/Documents%20and%20Settings/davris/FileSchemeURIs.doc', ur'C:\Documents and Settings\davris\FileSchemeURIs.doc')
            
            # improper syntax, but used anyway
            test_localFilePathOfUri(r'file://D:\Program Files\Viewer\startup.htm', ur'D:\Program Files\Viewer\startup.htm')
            test_localFilePathOfUri(r'C:\Program Files\Music\Web Sys\main.html?REQUEST=RADIO', ur'C:\Program Files\Music\Web Sys\main.html?REQUEST=RADIO')
            test_localFilePathOfUri(ur'c:\foo… \%20.txt', ur'c:\foo… \%20.txt')
            test_localFilePathOfUri(ur'\\My Computer\foo… \%20.txt', ur'\\My Computer\foo… \%20.txt')
            test_localFilePathOfUri(r'file:////applib/products/a%2Db/abc%5F9/4148.920a/media/start.swf', ur'\\applib\products\a-b\abc_9\4148.920a\media\start.swf')
            test_localFilePathOfUri(r'file:///C:/example%E3%84%93.txt', ur'C:\exampleㄓ.txt')
            test_localFilePathOfUri(ur'file:///C:/exampleㄓ.txt', ur'C:\exampleㄓ.txt')
            test_localFilePathOfUri(r'file://///localhost/transfer/test.txt', ur'\\localhost\transfer\test.txt')
            
            # invalid, but converted systematically
            test_localFilePathOfUri(r'file:///C:/?#<>:', u'C:\?#<>:')
            
        finally:
            _isWindows = platform.system() == 'Windows'        
        print "OK"
        
if __debug__ and __name__ == '__main__':
    _runTests()
