# -*- coding: utf-8 -*-
assert '…' == '\xe2\x80\xa6', "file encoding error"

# Copyright 2013 Axel Walthelm
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import re
import htmlentitydefs
import Utf8Tools as utf8

# Some tags like <p> break a word, i.e. they implicitly insert a space,
# while others like <i> do not.
# Similar to html classification as inline tag, but not the same.
NOSPACE_TAGS = frozenset([
    'a', 'abbr', 'acronym', 'b', 'basefont', 'bdo', 'big', 'cite',
    'code', 'dfn', 'em', 'font', 'i', 'img', 'input', 'kbd', 'label',
    'q', 's', 'samp', 'select', 'small', 'span', 'strike', 'strong',
    'sub', 'sup', 'textarea', 'tt', 'u', 'var',
    # mobipocket html extensions; guessing...
    'mbp:nu', 'idx:entry', 'idx:orth', 'idx:key', 'idx:short', 'idx:gramgrp', 'idx:subentry', 'idx:string', 'idx:ext-subentry' ])
    
# Some tags contain data that is not text
NOTEXT_TAGS = frozenset(['head', 'script', 'style', 'object', 'meta']) # tbd: more?

# The following are defined as inline elements in HTML 4:
INLINE_TAGS = frozenset([
    'a', 'abbr', 'acronym', 'b', 'basefont', 'bdo', 'big', 'br', 'cite',
    'code', 'dfn', 'em', 'font', 'i', 'img', 'input', 'kbd', 'label',
    'q', 's', 'samp', 'select', 'small', 'span', 'strike', 'strong',
    'sub', 'sup', 'textarea', 'tt', 'u', 'var' ])
                    
# The following elements may be used as either block-level elements or inline elements in HTML 4.
# If used as inline elements (e.g., within another inline element or a P),
# these elements should not contain any block-level elements.
INLINABLE_TAGS = frozenset([
    'applet', 'button', 'del', 'iframe', 'ins', 'map', 'object', 'script' ])

# The following are defined as block-level elements in HTML 4:
BLOCK_TAGS = frozenset([
    'address', 'blockquote', 'center', 'dir', 'div', 'dl', 'fieldset', 'form',
    'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'hr', 'isindex', 'menu', 'noframes',
    'noscript', 'ol', 'p', 'pre', 'table', 'ul',
    # The following elements may also be considered block-level elements since they may contain block-level elements:
    'dd', 'dt', 'frameset', 'li', 'tbody', 'td', 'tfoot', 'th', 'thead', 'tr' ])

__utf8Entities = None

def __makeUtf8Entities():
    entityDefs = { 'apos': "'" }
    for k, v in htmlentitydefs.name2codepoint.iteritems():
        entityDefs[k] = utf8.chr(v)
    return entityDefs

def utf8Entities():
    global __utf8Entities
    if __utf8Entities is None:
        __utf8Entities = __makeUtf8Entities()
    return __utf8Entities

def utf8HtmlNext(html, loc):
    if html[loc] == '&': # character is an entity like &uuml; => adjust loc to ; or next space
        while not re.match(r"[;\s]", html[loc]):
            loc += 1
        return loc+1
    return utf8.adjustR(html, loc+1);
