# -*- coding: utf-8 -*-
assert '…' == '\xe2\x80\xa6', "file encoding error"

# Copyright 2013 Axel Walthelm
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import re
import codecs

def readXmlDeclEnc(xml, default=None):
    # <?xml version='1.0' encoding='utf-8'?>
    match = re.match(br"\s*<\?*xml\s[^>]*\sencoding=[\"']?([^>?\"' ]*)", xml, re.IGNORECASE)
    return default if not match else match.group(1)

def checkXmlUtf8(xml, declEncoding=None):
    if not isinstance(xml, bytes):
        raise TypeError("XML not in bytes")
    if not declEncoding:
        return xml
    enc = readXmlDeclEnc(xml)
    if not enc or enc == declEncoding:
        return xml
    try:
        if codecs.lookup(enc).name == codecs.lookup(declEncoding).name:
            return xml
    except: pass
    raise AttributeError("XML encoding mismatch: %s declared as %s" % (declEncoding ,enc))

def makeXmlUtf8(xml):
    if isinstance(xml, unicode):
        return xml.encode('utf-8')
    if not isinstance(xml, bytes):
        raise TypeError()
    # detect encoding type
    enc = "utf-8", 0 # default: utf8, no BOM to skip
    #print "Xml startswith", xml[:4];
    if xml.startswith(b"\xef\xbb\xbf"): enc = "utf-8", 3 # utf-8 BOM
    # todo: test 16 and 32 bit
    elif xml.startswith(b"\x00\x00\xfe\xff"): enc = "utf-32-be", 4 # BOM
    elif xml.startswith(b"\xff\xfe\x00\x00"): enc = "utf-32-le", 4    
    elif xml.startswith(b"\xfe\xff"): enc = "utf-16-be", 2
    elif xml.startswith(b"\xff\xfe"): enc = "utf-16-le", 2
    elif re.match(br"\x00[\s<]", xml): enc = "utf-16-le", 0 # starts with < or (illegal) ascii-space
    elif re.match(br"\x00\x00\x00[\s<]", xml): enc = "utf-32-be", 0
    elif re.match(br"[\s<]\x00\x00\x00", xml): enc = "utf-32-le", 0
    elif re.match(br"[\s<]\x00", xml): enc = "utf-16-be", 0
    else: # probably some ascii based encoding => try look in xml declaration
        enc = readXmlDeclEnc(xml, 'utf-8'), 0
        #print "Xml heading", enc;
    # decode
    if enc[0] == "utf-8":
        return checkXmlUtf8( xml[enc[1]:], 'utf-8' )
    return checkXmlUtf8( xml[enc[1]:].decode(enc[0]).encode('utf-8') )
