#!/usr/bin/env python
# vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
from __future__ import (unicode_literals, division, absolute_import, print_function)

__license__   = 'GPL v3'
__copyright__ = '2013, Axel Walthelm'
__docformat__ = 'restructuredtext en'

from io import BytesIO
from gettext import (GNUTranslations, NullTranslations)
from calibre.utils.localization import get_lang

if False:
    # This is here to keep my python error checker from complaining about
    # the builtin functions that will be defined by the plugin loading system
    get_icons = get_resources = None

def _(msg):
    return msg

def _openMoFile():
    # get language used by Calibre
    language = get_lang()
    print("KiNotes: language =", language)

    # get best .mo file for language from resources
    # note: we can't use     
    #    import gettext
    #    moFile = gettext.find('KiNotes', localedir=r'D:\PythonWorkspace\KiNotes\locale', languages=[language])
    #    return file(moFile, 'rb')
    # because the plugin is run from the zip file and localedir does not exist in file system.
    #
    # instead we code a simplified mapping here, for which all existing .mo files must be known
    moFile = None
    if language.split('-',1)[0].lower() in ('de', 'nds'): # nds is Plattdeutsch (fallback: Deutsch)
        moFile = 'locale/de_DE/LC_MESSAGES/KiNotes.mo'
    print("KiNotes: moFile =", moFile)
    return BytesIO(get_resources(moFile))

try:
    with _openMoFile() as f:
        gettext = GNUTranslations(f)
    print("KiNotes: using gettext.GNUTranslations()")
except:
    #import traceback
    #traceback.print_exc()
    gettext = NullTranslations()
    print("KiNotes: using gettext.NullTranslations()")

_ = gettext.ugettext
